#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "Point.h"
#include "PointList.h"

int read_point(FILE *f, Point *p); 
int point_compare(const void* a1, const void* a2) ; 

PointList* read_list(char* filename); 
void print_loop(PointList* p); 





typedef struct 
{
  double 	angle;
  Point*	point;
}
AngleLocator;

int angle_compare(const void* p1, const void* p2)
{
  const AngleLocator* a1 = p1;
  const AngleLocator* a2 = p2;
  return a1->angle < a2->angle;
}

void sort_list_polar(PointList *p)
{
  double dx,dy;
  int i,n=0;
  Point	        *buf; 
  PointList	*start = p; 
  AngleLocator 	*a;

  if (p == NULL) return; 
  buf = p->point;

// How many points? 
  while (p!=NULL) { n++; p=p->next; } 

// Kill old list (except head point)


  a = malloc(sizeof(AngleLocator) * (n-1)); 
  for (i=1;i<n;i++)
  {
    dx = buf[i].x-buf[0].x;
    dy = buf[i].y-buf[0].y;
    a[i-1].angle = atan(dx/dy);
    a[i-1].point = buf+i;
  }
  qsort(a,n-1,sizeof(AngleLocator),angle_compare);

  i=0;
  p=start->next;
  while (p!=NULL)  // res
  {
    p->point=a[i].point;
    p=p->next; 
    i++; 
  }

  free(a);
  
}

// z-component of cross product at point p2
double make_cross(Point* p1, Point* p2, Point* p3)
{
  double dx12 = p1->x - p2->x; double dy12 = p1->y - p2->y;
  double dx23 = p2->x - p3->x; double dy23 = p2->y - p3->y;
 
  return (dx12 * dy23 - dy12 * dx23);
}

void make_hull(PointList* p)
{
  int flag=1;
  PointList *p1,*p2;
  if (p==NULL) return;

  p1 = p->next;
  if (p1!=NULL) p2=p1->next;

  do
  {
    if (make_cross(p->point,p1->point,p2->point) < 0.0) 
    {
      free(p1); 
      p->next = p2; p2->prev = p;  
// Go back, recheck previous!
      if (p->prev != NULL) { p=p->prev;} 
      else 
      { 
        fprintf(stderr,"error - fallen off start of list - quitting\n");
        exit(1);
      } 
    }
    else // Point survives for now, move on.
    {
      p=p1;
    }
    p1 = p->next; 
    if (p1 != NULL) p2 = p1->next; 
  }
  while (p2 != NULL); 
  
}

int main(int argc, char* argv[])
{
  PointList *l = read_list(argv[1]);

  sort_list_polar(l);

  print_loop(l);
  printf("@type xy\n");
  make_hull(l);
// list now holds just the convex hull
  print_loop(l);
}
